import { pagesVirtualModuleId, resolvedPagesVirtualModuleId } from "../../app/index.js";
import { addRollupInput } from "../add-rollup-input.js";
import { eachPageData } from "../internal.js";
import { MIDDLEWARE_MODULE_ID } from "./plugin-middleware.js";
function vitePluginPages(opts, internals) {
  return {
    name: "@astro/plugin-build-pages",
    options(options) {
      if (opts.settings.config.output === "static") {
        return addRollupInput(options, [pagesVirtualModuleId]);
      }
    },
    resolveId(id) {
      if (id === pagesVirtualModuleId) {
        return resolvedPagesVirtualModuleId;
      }
    },
    async load(id) {
      if (id === resolvedPagesVirtualModuleId) {
        let importMap = "";
        let imports = [];
        let i = 0;
        for (const pageData of eachPageData(internals)) {
          const variable = `_page${i}`;
          imports.push(
            `const ${variable} = () => import(${JSON.stringify(pageData.moduleSpecifier)});`
          );
          importMap += `[${JSON.stringify(pageData.component)}, ${variable}],`;
          i++;
        }
        i = 0;
        let rendererItems = "";
        for (const renderer of opts.settings.renderers) {
          const variable = `_renderer${i}`;
          imports.unshift(`import ${variable} from '${renderer.serverEntrypoint}';`);
          rendererItems += `Object.assign(${JSON.stringify(renderer)}, { ssr: ${variable} }),`;
          i++;
        }
        const def = `${imports.join("\n")}

${opts.settings.config.experimental.middleware ? `import * as _middleware from "${MIDDLEWARE_MODULE_ID}";` : ""}

export const pageMap = new Map([${importMap}]);
export const renderers = [${rendererItems}];
${opts.settings.config.experimental.middleware ? `export const middleware = _middleware;` : ""}
`;
        return def;
      }
    }
  };
}
function pluginPages(opts, internals) {
  return {
    build: "ssr",
    hooks: {
      "build:before": () => {
        return {
          vitePlugin: vitePluginPages(opts, internals)
        };
      }
    }
  };
}
export {
  pluginPages
};
